"""photodir.py - Generates a photo directory (from csv to tex to pdf)."""

# Copyright (C) 2011  Chris Mayfield <csmayfield@gmail.com>
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import csv
import os

from options import *


# LaTeX template (margins, formatting, layout, etc.)
PREAMBLE = r"""% THIS FILE WAS GENERATED BY photodir.py
\documentclass[10pt]{article}
\usepackage[margin=0.5in, paperwidth=5.5in, paperheight=8.5in]{geometry}
\usepackage{graphicx}
\usepackage{ifthen}

\graphicspath{{pics/}}
\setlength{\parindent}{0pt}
\pagestyle{empty}

\newcommand{\println}[1]{% string
\ifthenelse{\equal{#1}{}}{}{#1 \\}
}

% #1=family, #2=names, #3=photo
% #4=phone1, #5=phone2, #6=phone3, #7=addr1, #8=addr2

\newcommand{\extra}[8]{%
\textbf{#1: #2} \\*
\ifthenelse{\equal{#4}{}}{\ifthenelse{\equal{#5}{}}{#6}{#5}}{#4} ~~ #7#8 \\[4pt]
}

\newcommand{\entry}[8]{%
\begin{minipage}[b][2.45in][t]{2.15in}
\centering
\includegraphics[height=1.35in]{#3} \\
\small
\textbf{#1: #2} \\
\footnotesize
\smallskip
\println{#4}
\println{#5}
\println{#6}
\smallskip
\println{#7}
\println{#8}
\end{minipage}}

\newcommand{\blank}{%
\begin{minipage}[b][2.45in][t]{2.15in}
\end{minipage}}
"""


class Entry:

    def __init__(self, data):
        """Parses a household from a row of csv data."""

        # build list of family name and first names
        names = [data[0]]
        for i in range(5, len(data), 3):
            if data[i] != "":
                given = data[i].split(", ")[1]
                first = given.split(' ')[0]
                names.append(first)
        del i, given, first  # for debugger
        tweak_names(names)

        # get display names and photo file
        self.family = names[0]
        self.names = ", ".join(names[1:])
        if data[1].find(" & ") > -1:
            self.names = self.names.replace(", ", " \& ", 1)
        self.photo = names[0] + '_' + names[1] + ".jpg"
        self.photo = self.photo.replace(' ', '_')  # for LaTeX
        self.exists = os.path.exists("pics/" + self.photo)

        # get unique phone numbers
        self.phone1 = data[2]
        self.phone2 = data[6]
        self.phone3 = data[9]
        if self.phone1 == self.phone2 or self.phone1 == self.phone3:
            self.phone1 = ""
        if self.phone2 != "":
            self.phone2 += " (" + names[1] + ')'
        if self.phone3 != "":
            self.phone3 += " (" + names[2] + ')'

        # split address into two lines
        addr = data[4]
        for city in CITIES:
            ix = addr.rfind(city)
            if ix > -1:
                break
        if ix == -1:
            ix = addr.rfind(',')
        self.addr1 = addr[:ix].replace('#', "\\#")  # for LaTeX
        self.addr2 = addr[ix:]
        tweak_entry(self)

    def __cmp__(self, other):
        """Comparison function; sort by display names."""

        if self.family < other.family:
            return -1
        if self.family > other.family:
            return 1
        if self.names < other.names:
            return -1
        if self.names > other.names:
            return 1
        return 0

    def output(self, ofile, otype):
        """Outputs the resulting LaTeX entry."""

        ofile.write(otype)
        ofile.write("{" + self.family + "}\n")
        ofile.write("{" + self.names + "}\n")
        ofile.write("{" + self.photo + "}\n")
        ofile.write("{" + self.phone1 + "}\n")
        ofile.write("{" + self.phone2 + "}\n")
        ofile.write("{" + self.phone3 + "}\n")
        ofile.write("{" + self.addr1 + "}\n")
        ofile.write("{" + self.addr2 + "}\n")


CNT = 0  # how many entries/blanks have been output
def output_fill(ofile):
    global CNT
    CNT += 1

    # alternate hfill and vfill
    ofile.write("%\n")
    if CNT % 2 > 0:
        ofile.write("\\hfill\n")
    else:
        ofile.write("\\vfill\n")
    ofile.write("%\n")


def main():

    # parse households into a sorted list of objects
    ifile = open("households.csv", "rb")
    ifile.readline()  # csv header
    ents = [Entry(data) for data in csv.reader(ifile)]
    ents.sort()  # sometimes necessary
    ifile.close()

    # create the output file with optional front matter
    ofile = open("photodir.tex", 'w')
    ofile.write(PREAMBLE)
    ofile.write("\n\\begin{document}\n\n")
    if os.path.exists("cover.tex"):
        ofile.write("\\include{cover}\n\n")
    if os.path.exists("leadership.tex"):
        ofile.write("\\include{leadership}\n\n")

    # keep track of which photos are used
    pics = set(os.listdir("pics/"))
    npic = len(pics)

    # convert csv rows into tex entries
    for entry in ents:
        if PRUNE and not entry.exists:
            continue
        pics.discard(entry.photo)
        entry.output(ofile, "\\entry\n")
        output_fill(ofile)

    # fill the last page with blanks
    while CNT % 6 > 0:
        ofile.write("\\blank\n")
        output_fill(ofile)

	# create optional appendix
    if PRUNE and EXTRA:
        ofile.write("\n\\footnotesize\n\n")
        for entry in ents:
            if not entry.exists:
                entry.output(ofile, "\\extra\n")
                ofile.write("%\n")

    # close files and run LaTeX
    ofile.write("\n\\end{document}\n")
    ofile.close()
    os.system("pdflatex -interaction=nonstopmode photodir.tex")

    # output statistics
    print '\n\nSUMMARY:'
    print "  %d households" % len(ents)
    print "  %d with photos" % (npic - len(pics))
    if len(pics) > 0:
        print '\n\nUNUSED PICS:'
        for p in sorted(pics):
            print ' ', p
    print


if __name__ == "__main__":
    main()
